@@ -607,6 +607,11 @@ tp_filter_motion_unaccelerated(struct tp_dispatch *tp,
 			       const struct device_float_coords *unaccelerated,
 			       uint64_t time);
 
+struct normalized_coords
+tp_filter_scroll(struct tp_dispatch *tp,
+		 const struct device_float_coords *unaccelerated,
+		 uint64_t time);
+
 bool
 tp_touch_active(const struct tp_dispatch *tp, const struct tp_touch *t);
 
@@ -726,9 +731,6 @@ tp_gesture_stop_twofinger_scroll(struct tp_dispatch *tp, uint64_t time);
 void
 tp_gesture_tap_timeout(struct tp_dispatch *tp, uint64_t time);
 
-bool
-tp_palm_tap_is_palm(const struct tp_dispatch *tp, const struct tp_touch *t);
-
 void
 tp_clickpad_middlebutton_apply_config(struct evdev_device *device);
 
