@@ -66,3 +66,9 @@ input_event_set_time(struct input_event *e,
 	e->input_event_sec = tval.tv_sec;
 	e->input_event_usec = tval.tv_usec;
 }
+
+static inline double
+absinfo_range(const struct input_absinfo *abs)
+{
+	return (double)(abs->maximum - abs->minimum + 1);
+}
